@@ -20,6 +20,8 @@
  * OF THIS SOFTWARE.
  */
 
+#include "config.h"
+
 #include <assert.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -31,6 +33,7 @@
 #include <sys/time.h>
 #include <cairo.h>
 #include <sys/epoll.h>
+#include <stdbool.h>
 
 #include <wayland-client.h>
 #include <wayland-cursor.h>
@@ -40,6 +43,12 @@
 
 struct dnd_drag;
 
+struct pointer {
+	struct input *input;
+	bool dragging;
+	struct wl_list link;
+};
+
 struct dnd {
 	struct window *window;
 	struct widget *widget;
@@ -48,6 +57,7 @@ struct dnd {
 	struct item *items[16];
 	int self_only;
 	struct dnd_drag *current_drag;
+	struct wl_list pointers;
 };
 
 struct dnd_drag {
@@ -93,12 +103,12 @@ item_create(struct display *display, int x, int y, int seed)
 	item = malloc(sizeof *item);
 	if (item == NULL)
 		return NULL;
-	
-	
+
+
 	gettimeofday(&tv, NULL);
 	item->seed = seed ? seed : tv.tv_usec;
 	srandom(item->seed);
-	
+
 	const int petal_count = 3 + random() % 5;
 	const double r1 = 20 + random() % 10;
 	const double r2 = 5 + random() % 12;
@@ -279,7 +289,7 @@ static void
 data_source_send(void *data, struct wl_data_source *source,
 		 const char *mime_type, int32_t fd)
 {
-	struct dnd_flower_message dnd_flower_message;	
+	struct dnd_flower_message dnd_flower_message;
 	struct dnd_drag *dnd_drag = data;
 	char buffer[128];
 	int n;
@@ -315,7 +325,7 @@ data_source_cancelled(void *data, struct wl_data_source *source)
 	 * up the drag object created and the local state. */
 
 	wl_data_source_destroy(dnd_drag->data_source);
-	
+
 	/* Destroy the item that has been dragged out */
 	cairo_surface_destroy(dnd_drag->item->surface);
 	free(dnd_drag->item);
@@ -452,6 +462,40 @@ create_drag_source(struct dnd *dnd,
 		return -1;
 }
 
+static int
+lookup_cursor(struct dnd *dnd, int x, int y)
+{
+	struct item *item;
+
+	item = dnd_get_item(dnd, x, y);
+	if (item)
+		return CURSOR_HAND1;
+	else
+		return CURSOR_LEFT_PTR;
+}
+
+/* Update all the mouse pointers in the window appropriately.
+ * Optionally, skip one (which will be the current pointer just
+ * about to start a drag).  This is done here to save a scan
+ * through the pointer list.
+ */
+static void
+update_pointer_images_except(struct dnd *dnd, struct input *except)
+{
+	struct pointer *pointer;
+	int32_t x, y;
+
+	wl_list_for_each(pointer, &dnd->pointers, link) {
+		if (pointer->input == except) {
+			pointer->dragging = true;
+			continue;
+		}
+		input_get_position(pointer->input, &x, &y);
+		input_set_pointer_image(pointer->input,
+					lookup_cursor(dnd, x, y));
+	}
+}
+
 static void
 dnd_button_handler(struct widget *widget,
 		   struct input *input, uint32_t time,
@@ -464,8 +508,10 @@ dnd_button_handler(struct widget *widget,
 	input_get_position(input, &x, &y);
 	if (state == WL_POINTER_BUTTON_STATE_PRESSED) {
 		input_ungrab(input);
-		if (create_drag_source(dnd, input, time, x, y) == 0)
+		if (create_drag_source(dnd, input, time, x, y) == 0) {
 			input_set_pointer_image(input, CURSOR_DRAGGING);
+			update_pointer_images_except(dnd, input);
+		}
 	}
 }
 
@@ -487,34 +533,53 @@ dnd_touch_down_handler(struct widget *widget,
 		touch_grab(input, 0);
 }
 
-static int
-lookup_cursor(struct dnd *dnd, int x, int y)
-{
-	struct item *item;
-
-	item = dnd_get_item(dnd, x, y);
-	if (item)
-		return CURSOR_HAND1;
-	else
-		return CURSOR_LEFT_PTR;
-}
-
 static int
 dnd_enter_handler(struct widget *widget,
 		  struct input *input, float x, float y, void *data)
 {
 	struct dnd *dnd = data;
+	struct pointer *new_pointer = malloc(sizeof *new_pointer);
 
 	dnd->current_drag = NULL;
 
+	if (new_pointer) {
+		new_pointer->input = input;
+		new_pointer->dragging = false;
+		wl_list_insert(dnd->pointers.prev, &new_pointer->link);
+	}
+
 	return lookup_cursor(dnd, x, y);
 }
 
+static void
+dnd_leave_handler(struct widget *widget,
+		  struct input *input, void *data)
+{
+	struct dnd *dnd = data;
+	struct pointer *pointer, *tmp;
+
+	wl_list_for_each_safe(pointer, tmp, &dnd->pointers, link)
+		if (pointer->input == input) {
+			wl_list_remove(&pointer->link);
+			free(pointer);
+		}
+}
+
 static int
 dnd_motion_handler(struct widget *widget,
 		   struct input *input, uint32_t time,
 		   float x, float y, void *data)
 {
+	struct dnd *dnd = data;
+	struct pointer *pointer;
+
+	wl_list_for_each(pointer, &dnd->pointers, link)
+		if (pointer->input == input) {
+			if (pointer->dragging)
+				return CURSOR_DRAGGING;
+			break;
+		}
+
 	return lookup_cursor(data, x, y);
 }
 
@@ -554,7 +619,7 @@ dnd_receive_func(void *data, size_t len, int32_t x, int32_t y, void *user_data)
 			len, sizeof *message);
 		return;
 	}
-		
+
 	widget_get_allocation(dnd->widget, &allocation);
 	item = item_create(dnd->display,
 			   x - message->x_offset - allocation.x,
@@ -562,6 +627,7 @@ dnd_receive_func(void *data, size_t len, int32_t x, int32_t y, void *user_data)
 			   message->seed);
 
 	dnd_add_item(dnd, item);
+	update_pointer_images_except(dnd, NULL);
 	window_schedule_redraw(dnd->window);
 }
 
@@ -608,6 +674,8 @@ dnd_create(struct display *display)
 	dnd->display = display;
 	dnd->key = 100;
 
+	wl_list_init(&dnd->pointers);
+
 	for (i = 0; i < ARRAY_LENGTH(dnd->items); i++) {
 		x = (i % 4) * (item_width + item_padding) + item_padding;
 		y = (i / 4) * (item_height + item_padding) + item_padding;
@@ -625,6 +693,7 @@ dnd_create(struct display *display)
 
 	widget_set_redraw_handler(dnd->widget, dnd_redraw_handler);
 	widget_set_enter_handler(dnd->widget, dnd_enter_handler);
+	widget_set_leave_handler(dnd->widget, dnd_leave_handler);
 	widget_set_motion_handler(dnd->widget, dnd_motion_handler);
 	widget_set_button_handler(dnd->widget, dnd_button_handler);
 	widget_set_touch_down_handler(dnd->widget, dnd_touch_down_handler);
@@ -637,12 +706,27 @@ dnd_create(struct display *display)
 	return dnd;
 }
 
+static void
+dnd_destroy(struct dnd *dnd)
+{
+	widget_destroy(dnd->widget);
+	window_destroy(dnd->window);
+	free(dnd);
+}
+
 int
 main(int argc, char *argv[])
 {
 	struct display *d;
 	struct dnd *dnd;
-	int i;
+	int self_only = 0;
+
+	if (argc == 2 && !strcmp(argv[1], "--self-only"))
+		self_only = 1;
+	else if (argc > 1) {
+		printf("Usage: %s [OPTIONS]\n  --self-only\n", argv[0]);
+		return 1;
+	}
 
 	d = display_create(&argc, argv);
 	if (d == NULL) {
@@ -651,12 +735,13 @@ main(int argc, char *argv[])
 	}
 
 	dnd = dnd_create(d);
-
-	for (i = 1; i < argc; i++)
-		if (strcmp("--self-only", argv[i]) == 0)
-			dnd->self_only = 1;
+	if (self_only)
+		dnd->self_only = 1;
 
 	display_run(d);
 
+	dnd_destroy(dnd);
+	display_destroy(d);
+
 	return 0;
 }
