@@ -282,6 +282,30 @@ weston_config_section_get_bool(struct weston_config_section *section,
 	return 0;
 }
 
+WL_EXPORT
+const char *
+weston_config_get_libexec_dir(void)
+{
+	const char *path = getenv("WESTON_BUILD_DIR");
+
+	if (path)
+		return path;
+
+	return LIBEXECDIR;
+}
+
+const char *
+weston_config_get_name_from_env(void)
+{
+	const char *name;
+
+	name = getenv(WESTON_CONFIG_FILE_ENV_VAR);
+	if (name)
+		return name;
+
+	return "weston.ini";
+}
+
 static struct weston_config_section *
 config_add_section(struct weston_config *config, const char *name)
 {
@@ -314,6 +338,7 @@ weston_config_parse(const char *name)
 {
 	FILE *fp;
 	char line[512], *p;
+	struct stat filestat;
 	struct weston_config *config;
 	struct weston_config_section *section = NULL;
 	int i, fd;
@@ -330,6 +355,13 @@ weston_config_parse(const char *name)
 		return NULL;
 	}
 
+	if (fstat(fd, &filestat) < 0 ||
+	    !S_ISREG(filestat.st_mode)) {
+		close(fd);
+		free(config);
+		return NULL;
+	}
+
 	fp = fdopen(fd, "r");
 	if (fp == NULL) {
 		free(config);
